import logging
import logging.handlers
import coloredlogs
import os

logger = None
logging_to_file = False


__all__ = ['verbosity', 'save']


def _init():
    global logger

    logger = logging.getLogger('npy')
    logger.propagate = False

    logger.setLevel(logging.DEBUG)
    level_styles = {
                    'critical': {'color': 'red', 'bold': True},
                    'error': {'color': 'yellow'},
                    'warning': {'color': 'green', 'bold': True},
                    'info': {},
                    'debug': {'color': 'black', 'bright': True},
                    }
    field_styles = {
        'asctime': {'color': 'blue'},
        'levelname': {'color': 'yellow', 'faint': True}
    }
    coloredlogs.install(level='DEBUG', logger=logger,
                        fmt='[%(asctime)s] %(message)s',
                        datefmt="%m-%d %H:%M:%S",
                        field_styles=field_styles,
                        level_styles=level_styles)


def save():
    global logging_to_file

    if not logging_to_file:
        fmt = logging.Formatter('[%(asctime)s] %(message)s', "%m-%d %H:%M:%S")
        os.makedirs('log', exist_ok=True)
        fh = logging.handlers.TimedRotatingFileHandler('log/log.log', when='D')
        fh.setFormatter(fmt)
        logger.addHandler(fh)
        logging_to_file = True


def verbosity(level=2):
    """

    :param int level:
    :return:
    """
    if not isinstance(level, int):
        return

    if logger is None:
        _init()

    if level <= 1:
        logger.setLevel(logging.DEBUG)

    elif level <= 2:
        logger.setLevel(logging.INFO)

    elif level <= 3:
        logger.setLevel(logging.WARNING)

    elif level <= 4:
        logger.setLevel(logging.ERROR)

    else:
        logger.setLevel(logging.CRITICAL)


if logger is None:
    _init()
